//
//	jpeg.cpp - source file / freeware
//
//	David Henry - tfc_duke@club-internet.fr
//


#include	<stdio.h>

extern "C" {
#include	<jpeglib.h>
}

#ifdef _WIN32
#define	WIN32_LEAN_AND_MEAN
#include	<windows.h>
#endif

#include	<GL/gl.h>
#include	<iostream>

#include	"texture.h"


// Microsoft Visual C++
#ifdef _MSC_VER
#pragma comment( lib, "libjpeg.lib" )
#endif	// _MSC_VER



// --------------------------------------------------
// LoadFileJPEG() - charge une image JPEG. Si bFlipVert
// vaut 'true', l'image est retourne verticalement.
// --------------------------------------------------

bool LoadFileJPEG( const std::string &szFileName, gl_texture_t **img, bool bFlipVert )
{
	FILE *fp = NULL;				// pointeur fichier
	jpeg_decompress_struct cinfo;	// structure JPEG
	jpeg_error_mgr jerr;			// erreur lecture du fichier
	int rowsRead = 0;				// nombre de lignes de pixels lues
	GLubyte **row_pointers = NULL;	// tableau de pointeurs sur les lignes
									// de pixels de l'image


	// tentative d'ouverture du fichier en lecture
	if( !(fp = fopen( szFileName.c_str(), "rb" )) )
	{
		std::cout << "Error opening image file " << szFileName << std::endl;
		return false;
	}

	// lecture de l'ent-te et prparation  la
	// dcompression de l'image
	cinfo.err = jpeg_std_error( &jerr );
	jpeg_create_decompress( &cinfo );
	jpeg_stdio_src( &cinfo, fp );
	jpeg_read_header( &cinfo, TRUE );
	jpeg_start_decompress( &cinfo );

	// cre un objet texture
	*img = new gl_texture_t;
	(*img)->width = cinfo.image_width;
	(*img)->height = cinfo.image_height;
	(*img)->internalFormat = cinfo.num_components;

	if( cinfo.num_components == 1 ) {
		(*img)->format = GL_LUMINANCE;
	}
	else {
		(*img)->format = GL_RGB;
	}


	// allocation de mmoire pour les donnes pixels de l'image
	(*img)->texels = new GLubyte[(*img)->width * (*img)->height * (*img)->internalFormat];

	// cration d'un tableau de pointeurs sur les lignes de pixels de l'image
	row_pointers = new GLubyte*[ (*img)->height ];

	for( unsigned int i = 0; i < (*img)->height; i++ )
	{
		if( bFlipVert )
			row_pointers[i] = ((*img)->texels +
				(i * (*img)->width * (*img)->internalFormat));
		else
			row_pointers[i] = ((*img)->texels +
				(((*img)->height - (i + 1)) * (*img)->width * (*img)->internalFormat));
	}

	// extraction de chaque ligne de pixels de l'image
	while( cinfo.output_scanline < cinfo.output_height )
		rowsRead += jpeg_read_scanlines( &cinfo, &row_pointers[ rowsRead ], cinfo.output_height - rowsRead );

	// on a plus besoin des pointeurs sur les lignes
	delete [] row_pointers;

	// fin de la dcompression
	jpeg_finish_decompress( &cinfo );
	jpeg_destroy_decompress( &cinfo );

	fclose( fp );
	return true;
}
