//
//	png.cpp - source file / freeware
//
//	David Henry - tfc_duke@club-internet.fr
//


#ifdef _WIN32
#define	WIN32_LEAN_AND_MEAN
#include	<windows.h>
#endif

#include	<GL/gl.h>
#include	<iostream>
#include	<png.h>

#include	"texture.h"


// Microsoft Visual C++
#ifdef _MSC_VER
#pragma comment( lib, "libpng.lib" )
#pragma comment( lib, "zlib.lib" )
#endif	// _MSC_VER



// --------------------------------------------------
// LoadFilePNG() - charge une image PNG. Si bFlipVert
// vaut 'true', l'image est retourne verticalement.
// --------------------------------------------------

bool LoadFilePNG( const std::string &szFileName, gl_texture_t **img, bool bFlipVert )
{
	char			header[8];					// signature du fichier (numro magique)
	png_structp		png_ptr;					// structure PNG
	png_infop		info_ptr;					// infos image (caractristiques)
	int				bit_depth, color_type;		// profondeur en bits, type d'image PNG
	FILE 			*fp = NULL;					// pointeur fichier
	png_bytep		*row_pointers = NULL;		// tableau de pointeurs sur les lignes
												// de pixels de l'image

	// tentative d'ouverture du fichier en lecture
	if( !(fp = fopen( szFileName.c_str(), "rb" )) )
	{
		std::cout << "Error opening image file " << szFileName << std::endl;
		return false;
	}

	// lecture du numro magique
	fread( header, 1, sizeof( header ), fp );

	// vrifie que la signature est valide
	if( !png_check_sig( (unsigned char *)header, sizeof( header ) ) ) {
		std::cout << "Error: " << szFileName << "is not a PNG file" << std::endl;
		fclose( fp );
		return false; }

	if( !(png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL )) ) {
        fclose( fp );
		return false; }

    if( !(info_ptr = png_create_info_struct( png_ptr )) ) {
		fclose( fp );
		png_destroy_read_struct( &png_ptr, (png_infopp)NULL, (png_infopp)NULL );
		return false; }

	// cre un objet texture
	*img = new gl_texture_t;
	(*img)->format = GL_RGB;

	// lecture des infos de l'image dans info_ptr
	png_init_io( png_ptr, fp );
	png_set_sig_bytes( png_ptr, sizeof( header ) );
	png_read_info( png_ptr, info_ptr );

	(*img)->width = (GLuint)png_get_image_width( png_ptr, info_ptr );
	(*img)->height = (GLuint)png_get_image_height( png_ptr, info_ptr );
	bit_depth = (int)png_get_bit_depth( png_ptr, info_ptr );
	color_type = (int)png_get_color_type( png_ptr, info_ptr );

	// dtermine le format de texture OpenGL
	switch( color_type )
	{
		case PNG_COLOR_TYPE_GRAY:
			(*img)->format = GL_LUMINANCE;
			(*img)->internalFormat = 1;
			break;

		case PNG_COLOR_TYPE_GRAY_ALPHA:
			(*img)->format = GL_LUMINANCE_ALPHA;
			(*img)->internalFormat = 2;
			break;

		case PNG_COLOR_TYPE_RGB:
			(*img)->format = GL_RGB;
			(*img)->internalFormat = 3;
			break;

		case PNG_COLOR_TYPE_PALETTE:
			if( (int)info_ptr->trans ) {
				(*img)->format = GL_RGBA;
				(*img)->internalFormat = 4;
			}
			else {
				(*img)->format = GL_RGB;
				(*img)->internalFormat = 3;
			}

			break;

		case PNG_COLOR_TYPE_RGB_ALPHA:
			(*img)->format = GL_RGBA;
			(*img)->internalFormat = 4;
			break;

		default:
			// badness
			break;
	}

	// convertion des images index couleur en images RGB
	if( color_type == PNG_COLOR_TYPE_PALETTE )
		png_set_palette_to_rgb( png_ptr );

	if( color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8 )
		png_set_gray_1_2_4_to_8( png_ptr );

	if( png_get_valid( png_ptr, info_ptr, PNG_INFO_tRNS ) )
		png_set_tRNS_to_alpha( png_ptr );

	if( bit_depth == 16 )
		png_set_strip_16( png_ptr );
	else if( bit_depth < 8 )
		png_set_packing( png_ptr );

	// allocation de mmoire pour les donnes pixels de l'image
	(*img)->texels = new GLubyte[(*img)->width * (*img)->height * (*img)->internalFormat];

	// initialisation des pointeurs de chaque ligne de pixels sur
	// le tableau de texels de (*img)
	row_pointers = new png_bytep[ (*img)->height ];

	for( unsigned int i = 0; i < (*img)->height; i++ )
	{
		if( bFlipVert )
			row_pointers[i] = (png_bytep)((*img)->texels +
				(i * (*img)->width * (*img)->internalFormat));
		else
			row_pointers[i] = (png_bytep)((*img)->texels +
				(((*img)->height - (i + 1)) * (*img)->width * (*img)->internalFormat));
	}

	// lecture des donnes de l'image
	png_read_image( png_ptr, row_pointers );

	// on a plus besoin des pointeurs
	delete [] row_pointers;

	// libre la mmoire alloue pour les structures aprs la
	// lecture du fichier PNG
	png_read_end( png_ptr, NULL );
	png_destroy_read_struct( &png_ptr, &info_ptr, NULL );

	fclose( fp );
    return true;
}
