//
//	texture.h - header file
//
//	David Henry - tfc_duke@club-internet.fr
//


#ifndef		__TEXTURE_H
#define		__TEXTURE_H


#include	<map>
#include	<string>

#include	"singleton.h"



// objet texture OpenGL
typedef struct
{
	GLuint	width;
	GLuint	height;

	GLenum	format;
	GLint	internalFormat;
	GLuint	id;

	GLubyte	*texels;

} gl_texture_t;



// ==============================================
// CTextureManager - OpenGL texture manager.
// ==============================================

class CTextureManager : public CSingleton<CTextureManager>
{
	friend class CSingleton<CTextureManager>;

private:
	// constructeur/destructeur
	CTextureManager( void ) : m_iTextureCount(0) { m_kTextures.clear(); }
	~CTextureManager( void ) { ReleaseTextures(); }


public:
	// functions publiques
	bool	Initialize( void );
	void	ReleaseTextures( void );

	GLuint	LoadTexture( const std::string &szFileName, bool bFlipVert = false );

	gl_texture_t		*GetTexture( const std::string &szFileName );
	inline GLuint		GetTextureID( const std::string &szFileName );
	inline unsigned int	GetTextureCount( void ) const;

	inline bool	IsDefaultTexture( const gl_texture_t *img );
	inline bool	IsDefaultTexture( const std::string &szFileName );

private:
	// fonctions privées
	bool	LoadTextureFromFile( const std::string &szFileName, gl_texture_t **img, bool bFlipVert );
	void	CreateGLTexture( gl_texture_t *img );
	bool	CreateDefaultTexture( unsigned int size );


private:
	// texture map<>
	std::map<std::string, gl_texture_t*>::iterator	m_kItor;
	std::map<std::string, gl_texture_t*>			m_kTextures;

	gl_texture_t	*m_pDefaultTex;		// pointeur sur la texture 'default'
	unsigned int	m_iTextureCount;	// nombre de textures stockées

};



//
// définitions des fonctions membres inline
//

inline GLuint CTextureManager::GetTextureID( const std::string &szFileName )
{
	return GetTexture( szFileName )->id;
}

inline unsigned int CTextureManager::GetTextureCount( void ) const
{
	return m_iTextureCount;
}

inline bool CTextureManager::IsDefaultTexture( const gl_texture_t *img )
{
	return (img->id == m_pDefaultTex->id);
}

inline bool CTextureManager::IsDefaultTexture( const std::string &szFileName )
{
	return IsDefaultTexture( GetTexture( szFileName ) );
}



//
// prototypes des routines de lecture de fichiers images
//

bool LoadFilePNG( const std::string &szFileName, gl_texture_t **img, bool bFlipVert );
bool LoadFileJPEG( const std::string &szFileName, gl_texture_t **img, bool bFlipVert );
bool LoadFileTGA( const std::string &szFileName, gl_texture_t **img, bool bFlipVert );
bool LoadFileBMP( const std::string &szFileName, gl_texture_t **img, bool bFlipVert );
bool LoadFilePCX( const std::string &szFileName, gl_texture_t **img, bool bFlipVert );


#endif	// __TEXTURE_H
